# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>


from enum import Enum

from . import _ext

__all__ = ["Value", "Direction", "Bias", "Drive", "Edge", "Clock"]


class Value(Enum):
    """Logical line states."""

    INACTIVE = _ext.VALUE_INACTIVE
    """Line is logically inactive."""
    ACTIVE = _ext.VALUE_ACTIVE
    """Line is logically active."""

    def __bool__(self) -> bool:
        return self == self.ACTIVE


class Direction(Enum):
    """Direction settings."""

    AS_IS = _ext.DIRECTION_AS_IS
    """Request the line(s), but don't change direction."""
    INPUT = _ext.DIRECTION_INPUT
    """Direction is input - for reading the value of an externally driven GPIO line."""
    OUTPUT = _ext.DIRECTION_OUTPUT
    """Direction is output - for driving the GPIO line."""


class Bias(Enum):
    """Internal bias settings."""

    AS_IS = _ext.BIAS_AS_IS
    """Don't change the bias setting when applying line config."""
    UNKNOWN = _ext.BIAS_UNKNOWN
    """The internal bias state is unknown."""
    DISABLED = _ext.BIAS_DISABLED
    """The internal bias is disabled."""
    PULL_UP = _ext.BIAS_PULL_UP
    """The internal pull-up bias is enabled."""
    PULL_DOWN = _ext.BIAS_PULL_DOWN
    """The internal pull-down bias is enabled."""


class Drive(Enum):
    """Drive settings."""

    PUSH_PULL = _ext.DRIVE_PUSH_PULL
    """Drive setting is push-pull."""
    OPEN_DRAIN = _ext.DRIVE_OPEN_DRAIN
    """Line output is open-drain."""
    OPEN_SOURCE = _ext.DRIVE_OPEN_SOURCE
    """Line output is open-source."""


class Edge(Enum):
    """Edge detection settings."""

    NONE = _ext.EDGE_NONE
    """Line edge detection is disabled."""
    RISING = _ext.EDGE_RISING
    """Line detects rising edge events."""
    FALLING = _ext.EDGE_FALLING
    """Line detects falling edge events."""
    BOTH = _ext.EDGE_BOTH
    """Line detects both rising and falling edge events."""


class Clock(Enum):
    """Event clock settings."""

    MONOTONIC = _ext.CLOCK_MONOTONIC
    """Line uses the monotonic clock for edge event timestamps."""
    REALTIME = _ext.CLOCK_REALTIME
    """Line uses the realtime clock for edge event timestamps."""
    HTE = _ext.CLOCK_HTE
    """Line uses the hardware timestamp engine for event timestamps."""



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from dataclasses import dataclass
from enum import Enum

from . import _ext

__all__ = ["EdgeEvent"]


@dataclass(frozen=True, init=False, repr=False)
class EdgeEvent:
    """
    Immutable object containing data about a single edge event.

    Synchronization: none required
    """

    class Type(Enum):
        """Possible edge event types."""

        RISING_EDGE = _ext.EDGE_EVENT_TYPE_RISING
        """Rising edge event."""
        FALLING_EDGE = _ext.EDGE_EVENT_TYPE_FALLING
        """Falling edge event."""

    event_type: Type
    """Edge event type."""
    timestamp_ns: int
    """Timestamp of the event in nanoseconds."""
    line_offset: int
    """Offset of the line on which this event was registered."""
    global_seqno: int
    """Global sequence number of this event."""
    line_seqno: int
    """Event sequence number specific to the concerned line."""

    def __init__(
        self,
        event_type: int,
        timestamp_ns: int,
        line_offset: int,
        global_seqno: int,
        line_seqno: int,
    ):
        object.__setattr__(self, "event_type", EdgeEvent.Type(event_type))
        object.__setattr__(self, "timestamp_ns", timestamp_ns)
        object.__setattr__(self, "line_offset", line_offset)
        object.__setattr__(self, "global_seqno", global_seqno)
        object.__setattr__(self, "line_seqno", line_seqno)

    def __str__(self) -> str:
        return "<EdgeEvent type={} timestamp_ns={} line_offset={} global_seqno={} line_seqno={}>".format(  # noqa: UP032
            self.event_type,
            self.timestamp_ns,
            self.line_offset,
            self.global_seqno,
            self.line_seqno,
        )



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from __future__ import annotations

from datetime import timedelta
from select import select
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Generator, Iterable

    from .line_settings import LineSettings

__all__ = ["poll_fd", "config_iter"]


def poll_fd(fd: int, timeout: timedelta | float | None = None) -> bool:
    sec: float | None
    if isinstance(timeout, timedelta):
        sec = timeout.total_seconds()
    else:
        sec = timeout

    readable, _, _ = select([fd], [], [], sec)
    return True if fd in readable else False


def config_iter(
    config: dict[Iterable[int | str] | int | str, LineSettings | None],
) -> Generator[tuple[int | str, LineSettings | None]]:
    for key, settings in config.items():
        if isinstance(key, int) or isinstance(key, str):
            yield key, settings
        else:
            for subkey in key:
                yield subkey, settings



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

"""
Python bindings for libgpiod.

This module wraps the native C API of libgpiod in a set of python classes.
"""

from collections.abc import Iterable

from . import (
    _ext,
    chip,
    chip_info,
    edge_event,
    exception,
    info_event,
    line,
    line_info,
    line_request,
    line_settings,
    version,
)
from .chip import *
from .chip_info import *
from .edge_event import *
from .exception import *
from .info_event import *
from .line_info import *
from .line_request import *
from .line_settings import *
from .version import __version__

api_version = _ext.api_version

# public submodules
__all__ = [
    "chip",
    "chip_info",
    "edge_event",
    "exception",
    "info_event",
    "line",
    "line_info",
    "line_request",
    "line_settings",
    "version",
]

# re-export public submodule exports
# do not re-export line objects, this is not an oversight
__all__ += (
    chip.__all__
    + chip_info.__all__
    + edge_event.__all__
    + exception.__all__
    + info_event.__all__
    + line_info.__all__
    + line_request.__all__
    + line_settings.__all__
)

# module methods/attributes
__all__ += [
    "__version__",
    "api_version",
    "is_gpiochip_device",
    "request_lines",
]


def is_gpiochip_device(path: str) -> bool:
    """
    Check if the file pointed to by path is a GPIO chip character device.

    Args:
      path
        Path to the file that should be checked.

    Returns:
      Returns true if so, False otherwise.
    """
    return _ext.is_gpiochip_device(path)


def request_lines(
    path: str,
    config: dict[Iterable[int | str] | int | str, LineSettings | None],
    consumer: str | None = None,
    event_buffer_size: int | None = None,
    output_values: dict[int | str, line.Value] | None = None,
) -> LineRequest:
    """
    Open a GPIO chip pointed to by 'path', request lines according to the
    configuration arguments, close the chip and return the request object.

    Args:
      path
        Path to the GPIO character device file.
      config:
        Dictionary mapping offsets or names (or tuples thereof) to
        LineSettings. If None is passed as the value of the mapping,
        default settings are used.
      consumer:
        Consumer string to use for this request.
      event_buffer_size:
        Size of the kernel edge event buffer to configure for this request.
      output_values:
        Dictionary mapping offsets or names to line.Value. This can be used
        to set the desired output values globally while reusing LineSettings
        for more lines.

    Returns:
      Returns a new LineRequest object.
    """
    with Chip(path) as chip:
        return chip.request_lines(
            config=config,
            consumer=consumer,
            event_buffer_size=event_buffer_size,
            output_values=output_values,
        )



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>


from dataclasses import dataclass

__all__ = ["ChipInfo"]


@dataclass(frozen=True, repr=False)
class ChipInfo:
    """
    Immutable snapshot of a chip's status.

    Synchronization: none required
    """

    name: str
    """Name of the chip."""

    label: str
    """Label of the chip."""

    num_lines: int
    """Number of lines exposed by the chip."""

    def __str__(self) -> str:
        return f'<ChipInfo name="{self.name}" label="{self.label}" num_lines={self.num_lines}>'



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

__all__ = ["ChipClosedError", "RequestReleasedError"]


class ChipClosedError(Exception):
    """
    Error raised when an already closed chip is used.
    """

    def __init__(self) -> None:
        super().__init__("I/O operation on closed chip")


class RequestReleasedError(Exception):
    """
    Error raised when a released request is used.
    """

    def __init__(self) -> None:
        super().__init__("GPIO lines have been released")



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from dataclasses import dataclass
from datetime import timedelta

from .line import Bias, Clock, Direction, Drive, Edge

__all__ = ["LineInfo"]


@dataclass(frozen=True, init=False, repr=False)
class LineInfo:
    """
    Immutable snapshot of a line's status.

    Synchronization: none required
    """

    offset: int
    """Offset of the line."""
    name: str
    """Name of the line."""
    used: bool
    """Indicates whether line is in use."""
    consumer: str
    """Name of the consumer of the line."""
    direction: Direction
    """Direction setting of the line."""
    active_low: bool
    """Active-low setting of the line."""
    bias: Bias
    """Bias setting of the line."""
    drive: Drive
    """Drive setting of the line."""
    edge_detection: Edge
    """Edge detection setting of the line."""
    event_clock: Clock
    """Event clock setting used for edge event timestamps for the line."""
    debounced: bool
    """Indicates whether line is debounced."""
    debounce_period: timedelta
    """Debounce period of the line."""

    def __init__(
        self,
        offset: int,
        name: str,
        used: bool,
        consumer: str,
        direction: int,
        active_low: bool,
        bias: int,
        drive: int,
        edge_detection: int,
        event_clock: int,
        debounced: bool,
        debounce_period_us: int,
    ):
        object.__setattr__(self, "offset", offset)
        object.__setattr__(self, "name", name)
        object.__setattr__(self, "used", used)
        object.__setattr__(self, "consumer", consumer)
        object.__setattr__(self, "direction", Direction(direction))
        object.__setattr__(self, "active_low", active_low)
        object.__setattr__(self, "bias", Bias(bias))
        object.__setattr__(self, "drive", Drive(drive))
        object.__setattr__(self, "edge_detection", Edge(edge_detection))
        object.__setattr__(self, "event_clock", Clock(event_clock))
        object.__setattr__(self, "debounced", debounced)
        object.__setattr__(
            self, "debounce_period", timedelta(microseconds=debounce_period_us)
        )

    def __str__(self) -> str:
        return '<LineInfo offset={} name="{}" used={} consumer="{}" direction={} active_low={} bias={} drive={} edge_detection={} event_clock={} debounced={} debounce_period={}>'.format(  # noqa: UP032
            self.offset,
            self.name,
            self.used,
            self.consumer,
            self.direction,
            self.active_low,
            self.bias,
            self.drive,
            self.edge_detection,
            self.event_clock,
            self.debounced,
            self.debounce_period,
        )



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from dataclasses import dataclass
from datetime import timedelta

from . import _ext
from .line import Bias, Clock, Direction, Drive, Edge, Value

__all__ = ["LineSettings"]


@dataclass(repr=False)
class LineSettings:
    """
    Stores a set of line properties.

    Synchronization: none required
    """

    direction: Direction = Direction.AS_IS
    """Line direction."""
    edge_detection: Edge = Edge.NONE
    """Edge detection setting."""
    bias: Bias = Bias.AS_IS
    """Line bias setting."""
    drive: Drive = Drive.PUSH_PULL
    """Line drive setting."""
    active_low: bool = False
    """Active-low switch."""
    debounce_period: timedelta = timedelta()
    """Debounce period of the line."""
    event_clock: Clock = Clock.MONOTONIC
    """Edge event timestamping clock setting."""
    output_value: Value = Value.INACTIVE
    """Output value of the line."""

    # __repr__ generated by @dataclass uses repr for enum members resulting in
    # an unusable representation as those are of the form: <NAME: $value>
    def __repr__(self) -> str:
        return "gpiod.LineSettings(direction=gpiod.line.{}, edge_detection=gpiod.line.{}, bias=gpiod.line.{}, drive=gpiod.line.{}, active_low={}, debounce_period={}, event_clock=gpiod.line.{}, output_value=gpiod.line.{})".format(  # noqa: UP032
            str(self.direction),
            str(self.edge_detection),
            str(self.bias),
            str(self.drive),
            self.active_low,
            repr(self.debounce_period),
            str(self.event_clock),
            str(self.output_value),
        )

    def __str__(self) -> str:
        return "<LineSettings direction={} edge_detection={} bias={} drive={} active_low={} debounce_period={} event_clock={} output_value={}>".format(  # noqa: UP032
            self.direction,
            self.edge_detection,
            self.bias,
            self.drive,
            self.active_low,
            self.debounce_period,
            self.event_clock,
            self.output_value,
        )


def _line_settings_to_ext(settings: LineSettings) -> _ext.LineSettings:
    return _ext.LineSettings(
        direction=settings.direction.value,
        edge_detection=settings.edge_detection.value,
        bias=settings.bias.value,
        drive=settings.drive.value,
        active_low=settings.active_low,
        debounce_period=int(settings.debounce_period.total_seconds() * 1000000),
        event_clock=settings.event_clock.value,
        output_value=settings.output_value.value,
    )



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, cast

from . import _ext
from ._internal import config_iter, poll_fd
from .exception import RequestReleasedError
from .line_settings import LineSettings, _line_settings_to_ext

if TYPE_CHECKING:
    from collections.abc import Iterable
    from datetime import timedelta
    from types import TracebackType

    from .edge_event import EdgeEvent
    from .line import Value


__all__ = ["LineRequest"]


class LineRequest:
    """
    Stores the context of a set of requested GPIO lines.

    Synchronization: objects of this class require external synchronization.
    Protect calls with a lock when sharing an instance across threads.
    """

    def __init__(self, req: _ext.Request):
        """
        Note: LineRequest objects can only be instantiated by a Chip parent.
        LineRequest.__init__() is not part of stable API.
        """
        self._req: _ext.Request | None = req
        self._chip_name: str
        self._offsets: list[int]
        self._name_map: dict[str, int]
        self._lines: list[int | str]

    def __bool__(self) -> bool:
        """
        Boolean conversion for GPIO line requests.

        Returns:
          True if the request is live and False if it's been released.
        """
        return True if self._req else False

    def __enter__(self) -> LineRequest:
        """
        Controlled execution enter callback.
        """
        self._check_released()
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None:
        """
        Controlled execution exit callback.
        """
        self.release()

    def _check_released(self) -> None:
        if not self._req:
            raise RequestReleasedError()

    def release(self) -> None:
        """
        Release this request and free all associated resources. The object must
        not be used after a call to this method.
        """
        if self._req is not None:
            self._req.release()
            self._req = None

    def get_value(self, line: int | str) -> Value:
        """
        Get a single GPIO line value.

        Args:
          line:
            Offset or name of the line to get value for.

        Returns:
          Logical value of the line.
        """
        return self.get_values([line])[0]

    def _line_to_offset(self, line: int | str) -> int:
        if isinstance(line, int):
            return line
        if (_line := self._name_map.get(line)) is None:
            raise ValueError(f"unknown line name: {line}")
        return _line

    def get_values(self, lines: Iterable[int | str] | None = None) -> list[Value]:
        """
        Get values of a set of GPIO lines.

        Args:
          lines:
            List of names or offsets of GPIO lines to get values for. Can be
            None in which case all requested lines will be read.

        Returns:
          List of logical line values.
        """
        self._check_released()

        lines = lines or self._lines

        offsets = [self._line_to_offset(line) for line in lines]

        buf = cast("list[Value]", [None] * len(offsets))

        cast("_ext.Request", self._req).get_values(offsets, buf)
        return buf

    def set_value(self, line: int | str, value: Value) -> None:
        """
        Set the value of a single GPIO line.

        Args:
          line:
            Offset or name of the line to set.
          value:
            New value.
        """
        self.set_values({line: value})

    def set_values(self, values: dict[int | str, Value]) -> None:
        """
        Set the values of a subset of GPIO lines.

        Args:
          values:
            Dictionary mapping line offsets or names to desired values.
        """
        self._check_released()

        mapped = {self._line_to_offset(line): value for line, value in values.items()}

        cast("_ext.Request", self._req).set_values(mapped)

    def reconfigure_lines(
        self,
        config: dict[Iterable[int | str] | int | str, LineSettings | None],
    ) -> None:
        """
        Reconfigure requested lines.

        Args:
          config
            Dictionary mapping offsets or names (or tuples thereof) to
            LineSettings. If no entry exists, or a None is passed as the
            settings, then the configuration for that line is not changed.
            Any settings for non-requested lines are ignored.
        """
        self._check_released()

        line_cfg = _ext.LineConfig()
        line_settings = {}

        for line, settings in config_iter(config):
            try:
                offset = self._line_to_offset(line)
                if offset in self.offsets:
                    line_settings[offset] = settings
                else:
                    warnings.warn(
                        f"Line offset '{offset}' was not included in original request.",
                        stacklevel=2,
                    )
            except ValueError:
                # _line_to_offset will raise a ValueError when it encounters
                # an unrecognized line name. Ignore these like we do offsets
                # that were not in the original request.
                warnings.warn(
                    f"Line name '{line}' was not included in original request.",
                    stacklevel=2,
                )

        for offset in self.offsets:
            settings = line_settings.get(offset) or LineSettings()
            line_cfg.add_line_settings([offset], _line_settings_to_ext(settings))

        cast("_ext.Request", self._req).reconfigure_lines(line_cfg)

    def wait_edge_events(self, timeout: timedelta | float | None = None) -> bool:
        """
        Wait for edge events on any of the requested lines.

        Args:
          timeout:
            Wait time limit expressed as either a datetime.timedelta object
            or the number of seconds stored in a float. If set to 0, the
            method returns immediately, if set to None it blocks indefinitely.

        Returns:
          True if events are ready to be read. False on timeout.
        """
        self._check_released()

        return poll_fd(self.fd, timeout)

    def read_edge_events(self, max_events: int | None = None) -> list[EdgeEvent]:
        """
        Read a number of edge events from a line request.

        Args:
          max_events:
            Maximum number of events to read.

        Returns:
          List of read EdgeEvent objects.
        """
        self._check_released()

        return cast("_ext.Request", self._req).read_edge_events(max_events)

    def fileno(self) -> int:
        """
        Return the underlying file descriptor.
        """
        return self.fd

    def __str__(self) -> str:
        """
        Return a user-friendly, human-readable description of this request.
        """
        if not self._req:
            return "<LineRequest RELEASED>"

        return f'<LineRequest chip="{self.chip_name}" num_lines={self.num_lines} offsets={self.offsets} fd={self.fd}>'

    @property
    def chip_name(self) -> str:
        """
        Name of the chip this request was made on.
        """
        self._check_released()
        return self._chip_name

    @property
    def num_lines(self) -> int:
        """
        Number of requested lines.
        """
        self._check_released()
        return len(self._offsets)

    @property
    def offsets(self) -> list[int]:
        """
        List of requested offsets. Lines requested by name are mapped to their
        offsets.
        """
        self._check_released()
        return self._offsets

    @property
    def lines(self) -> list[int | str]:
        """
        List of requested lines. Lines requested by name are listed as such.
        """
        self._check_released()
        return self._lines

    @property
    def fd(self) -> int:
        """
        File descriptor associated with this request.
        """
        self._check_released()
        return cast("_ext.Request", self._req).fd



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2024 Vincent Fazio <vfazio@gmail.com>

from .chip_info import ChipInfo
from .edge_event import EdgeEvent
from .info_event import InfoEvent
from .line import Value
from .line_info import LineInfo

class LineSettings:
    def __init__(
        self,
        direction: int,
        edge_detection: int,
        bias: int,
        drive: int,
        active_low: bool,
        debounce_period: int,
        event_clock: int,
        output_value: int,
    ) -> None: ...

class LineConfig:
    def __init__(self) -> None: ...
    def add_line_settings(self, offsets: list[int], settings: LineSettings) -> None: ...
    def set_output_values(self, global_output_values: list[Value]) -> None: ...

class Request:
    def release(self) -> None: ...
    def get_values(self, offsets: list[int], values: list[Value]) -> None: ...
    def set_values(self, values: dict[int, Value]) -> None: ...
    def reconfigure_lines(self, line_cfg: LineConfig) -> None: ...
    def read_edge_events(self, max_events: int | None) -> list[EdgeEvent]: ...
    @property
    def chip_name(self) -> str: ...
    @property
    def num_lines(self) -> int: ...
    @property
    def offsets(self) -> list[int]: ...
    @property
    def fd(self) -> int: ...

class Chip:
    def __init__(self, path: str) -> None: ...
    def get_info(self) -> ChipInfo: ...
    def line_offset_from_id(self, id: str) -> int: ...
    def get_line_info(self, offset: int, watch: bool) -> LineInfo: ...
    def get_line_name(self, offset: int) -> str | None: ...
    def request_lines(
        self,
        line_cfg: LineConfig,
        consumer: str | None,
        event_buffer_size: int | None,
    ) -> Request: ...
    def read_info_event(self) -> InfoEvent: ...
    def close(self) -> None: ...
    def unwatch_line_info(self, line: int) -> None: ...
    @property
    def path(self) -> str: ...
    @property
    def fd(self) -> int: ...

def is_gpiochip_device(path: str) -> bool: ...

api_version: str

# enum constants
BIAS_AS_IS: int
BIAS_DISABLED: int
BIAS_PULL_DOWN: int
BIAS_PULL_UP: int
BIAS_UNKNOWN: int
CLOCK_HTE: int
CLOCK_MONOTONIC: int
CLOCK_REALTIME: int
DIRECTION_AS_IS: int
DIRECTION_INPUT: int
DIRECTION_OUTPUT: int
DRIVE_OPEN_DRAIN: int
DRIVE_OPEN_SOURCE: int
DRIVE_PUSH_PULL: int
EDGE_BOTH: int
EDGE_EVENT_TYPE_FALLING: int
EDGE_EVENT_TYPE_RISING: int
EDGE_FALLING: int
EDGE_NONE: int
EDGE_RISING: int
INFO_EVENT_TYPE_LINE_CONFIG_CHANGED: int
INFO_EVENT_TYPE_LINE_RELEASED: int
INFO_EVENT_TYPE_LINE_REQUESTED: int
VALUE_ACTIVE: int
VALUE_INACTIVE: int



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from dataclasses import dataclass
from enum import Enum

from . import _ext
from .line_info import LineInfo

__all__ = ["InfoEvent"]


@dataclass(frozen=True, init=False, repr=False)
class InfoEvent:
    """
    Immutable object containing data about a single line info event.

    Synchronization: none required
    """

    class Type(Enum):
        """Line status change event types."""

        LINE_REQUESTED = _ext.INFO_EVENT_TYPE_LINE_REQUESTED
        """Line has been requested."""
        LINE_RELEASED = _ext.INFO_EVENT_TYPE_LINE_RELEASED
        """Previously requested line has been released."""
        LINE_CONFIG_CHANGED = _ext.INFO_EVENT_TYPE_LINE_CONFIG_CHANGED
        """Line configuration has changed."""

    event_type: Type
    """Event type of the status change event."""
    timestamp_ns: int
    """Timestamp of the event."""
    line_info: LineInfo
    """Snapshot of line-info associated with the event."""

    def __init__(self, event_type: int, timestamp_ns: int, line_info: LineInfo):
        object.__setattr__(self, "event_type", InfoEvent.Type(event_type))
        object.__setattr__(self, "timestamp_ns", timestamp_ns)
        object.__setattr__(self, "line_info", line_info)

    def __str__(self) -> str:
        return f"<InfoEvent type={self.event_type} timestamp_ns={self.timestamp_ns} line_info={self.line_info}>"
